# coding=utf-8
"""
django-roughpages
"""
__author__ = 'Alisue <lambdalisue@hashnote.net>'
from app_version import get_versions
__version__, VERSION = get_versions('django-roughpages', allow_ambiguous=True)



# coding=utf-8
"""
Roughpage views
"""
__author__ = 'Alisue <lambdalisue@hashnote.net>'
import os
from django.http import Http404, HttpResponse
from django.template import loader, RequestContext, TemplateDoesNotExist
from django.shortcuts import redirect
from django.views.decorators.csrf import csrf_protect
from roughpages.conf import settings
from roughpages.backends import get_backend
from roughpages.utils import url_to_filename

# This view is called from RoughpageFallbackMiddleware.process_response
# when a 404 is raised, which often means CsrfViewMiddleware.process_view
# has not been called even if CsrfViewMiddleware is installed. So we need
# to use @csrf_protect, in case the template needs {% csrf_token %}.
# However, we can't just wrap this view; if no matching roughpage exists,
# or a redirect is required for authentication, the 404 needs to be returned
# without any CSRF checks. Therefore, we only
# CSRF protect the internal implementation.


def roughpage(request, url):
    """
    Public interface to the rough page view.
    """
    if settings.APPEND_SLASH and not url.endswith('/'):
        # redirect to the url which have end slash
        return redirect(url + '/', permanent=True)
    # get base filename from url
    filename = url_to_filename(url) + settings.ROUGHPAGES_TEMPLATE_FILE_EXT
    # add extra prefix path
    filename = os.path.join(settings.ROUGHPAGES_TEMPLATE_DIR, filename)
    # try to find the template_filename with backends
    template_filenames = get_backend().prepare_filenames(filename,
                                                         request=request)
    try:
        t = loader.select_template(template_filenames)
        return render_roughpage(request, t)
    except TemplateDoesNotExist:
        if settings.ROUGHPAGES_RAISE_TEMPLATE_DOES_NOT_EXISTS:
            raise
        raise Http404


@csrf_protect
def render_roughpage(request, t):
    """
    Internal interface to the rough page view.
    """
    c = RequestContext(request)
    response = HttpResponse(t.render(c))
    return response



# coding=utf-8
"""
Roughpage template filename backends
"""
__author__ = 'Alisue <lambdalisue@hashnote.net>'
import os


def get_backend(backend_class=None):
    from roughpages.conf import settings
    from roughpages.compat import import_module
    cache_name = '_backend_instance'
    if not hasattr(get_backend, cache_name):
        backend_class = backend_class or settings.ROUGHPAGES_BACKEND
        if isinstance(backend_class, basestring):
            module_path, class_name = backend_class.rsplit(".", 1)
            module = import_module(module_path)
            backend_class = getattr(module, class_name)
        setattr(get_backend, cache_name, backend_class())
    return getattr(get_backend, cache_name)


class TemplateFilenameBackendBase(object):
    """
    A base class of TemplateFilenameBackend
    """
    def prepare_filenames(self, filename, request):
        """
        Prepare template filename list

        Args:
            filename (str): A base filename
            request (instance): An instance of HttpRequest

        Returns:
            list

        Raises:
            NotImplementedError
        """
        raise NotImplementedError(
            "Subclass of TemplateFilenameBackendBase need to "
            "override 'prepare_filename' method.")


class PlainTemplateFilenameBackend(TemplateFilenameBackendBase):
    """
    A TemplateFilenameBackend which simply return filename single list
    """
    def prepare_filenames(self, filename, request):
        """
        Prepare template filename list

        Args:
            filename (str): A base filename
            request (instance): An instance of HttpRequest

        Returns:
            list

        Examples:
            >>> from mock import MagicMock
            >>> request = MagicMock()
            >>> backend = PlainTemplateFilenameBackend()
            >>> filenames = backend.prepare_filenames('foo/bar/hogehoge.html',
            ...                                       request)
            >>> assert filenames == [
            ...     'foo/bar/hogehoge.html'
            ... ]
        """
        return [filename]


class AuthTemplateFilenameBackend(TemplateFilenameBackendBase):
    """
    A TemplateFilenameBackend which return filename with authenticated state
    suffix.
    """
    def prepare_filenames(self, filename, request):
        """
        Prepare template filename list based on the user authenticated state

        If user is authenticated user, it use '_authenticated' as a suffix.
        Otherwise it use '_anonymous' as a suffix to produce the template
        filename list. The list include original filename at the end of the
        list.

        Args:
            filename (str): A base filename
            request (instance): An instance of HttpRequest

        Returns:
            list

        Examples:
            >>> from mock import MagicMock
            >>> request = MagicMock()
            >>> request.user.is_authenticated.return_value = True
            >>> backend = AuthTemplateFilenameBackend()
            >>> filenames = backend.prepare_filenames('foo/bar/hogehoge.html',
            ...                                       request)
            >>> assert filenames == [
            ...     'foo/bar/hogehoge_authenticated.html',
            ...     'foo/bar/hogehoge.html'
            ... ]
            >>> request.user.is_authenticated.return_value = False
            >>> filenames = backend.prepare_filenames('foo/bar/hogehoge.html',
            ...                                       request)
            >>> assert filenames == [
            ...     'foo/bar/hogehoge_anonymous.html',
            ...     'foo/bar/hogehoge.html'
            ... ]
        """
        basename, ext = os.path.splitext(filename)
        filenames = [filename]
        if request.user.is_authenticated():
            filenames.insert(0, "%s_authenticated%s" % (basename, ext))
        else:
            filenames.insert(0, "%s_anonymous%s" % (basename, ext))
        return filenames


if __name__ == '__main__':
    import doctest
    doctest.testmod()



# coding=utf-8
"""
"""
__author__ = 'Alisue <lambdalisue@hashnote.net>'
from django.test import TestCase
from roughpages import backends
from roughpages.tests.compat import MagicMock, override_settings

TEMPLATE_FILENAME = 'foo/bar/hogehoge.html'
ANONYMOUS_TEMPLATE_FILENAME = 'foo/bar/hogehoge_anonymous.html'
AUTHENTICATED_TEMPLATE_FILENAME = 'foo/bar/hogehoge_authenticated.html'

ANONYMOUS_REQUEST = MagicMock()
ANONYMOUS_REQUEST.user.is_authenticated.return_value = False
AUTHENTICATED_REQUEST = MagicMock()
AUTHENTICATED_REQUEST.user.is_authenticated.return_value = True


@override_settings(
    ROUGHPAGES_BACKEND='roughpages.backends.AuthTemplateFilenameBackend',
)
class RoughpagesBackendsTestCase(TestCase):
    def setUp(self):
        # clear cache
        if hasattr(backends.get_backend, '_backend_instance'):
            del backends.get_backend._backend_instance

    def test_get_backend(self):
        backend = backends.get_backend()
        self.assertTrue(isinstance(backend,
                                   backends.AuthTemplateFilenameBackend))

    def test_get_backend_with_string(self):
        backend = backends.get_backend(
            'roughpages.backends.PlainTemplateFilenameBackend')
        self.assertTrue(isinstance(backend,
                                   backends.PlainTemplateFilenameBackend))

    def test_get_backend_with_instance(self):
        backend = backends.get_backend(backends.PlainTemplateFilenameBackend)
        self.assertTrue(isinstance(backend,
                                   backends.PlainTemplateFilenameBackend))


class RoughpagesTemplateFilenameBackendBaseTestCase(TestCase):
    def setUp(self):
        self.backend = backends.TemplateFilenameBackendBase()
        self.request = MagicMock()

    def test_prepare_filenames_raise_exception(self):
        """prepare_filenames should raise NotImplementedError"""
        self.assertRaises(NotImplementedError,
                          self.backend.prepare_filenames,
                          TEMPLATE_FILENAME,
                          AUTHENTICATED_REQUEST)


class RoughpagesPlainTemplateFilenameBackendTestCase(TestCase):
    def setUp(self):
        self.backend = backends.PlainTemplateFilenameBackend()

    def test_prepare_filenames_return_correct_list(self):
        """prepare_filenames should return a list with original filename"""
        r = self.backend.prepare_filenames(TEMPLATE_FILENAME,
                                           AUTHENTICATED_REQUEST)
        self.assertEqual(r, [TEMPLATE_FILENAME])


class RoughpagesAuthTemplateFilenameBackendTestCase(TestCase):
    def setUp(self):
        self.backend = backends.AuthTemplateFilenameBackend()

    def test_prepare_filenames_with_anonymous(self):
        """prepare_filenames should return a list for anonymous user"""
        r = self.backend.prepare_filenames(TEMPLATE_FILENAME,
                                           ANONYMOUS_REQUEST)
        self.assertEqual(r, [
            ANONYMOUS_TEMPLATE_FILENAME,
            TEMPLATE_FILENAME,
        ])

    def test_prepare_filenames_with_authenticated(self):
        """prepare_filenames should return a list for authenticated user"""
        r = self.backend.prepare_filenames(TEMPLATE_FILENAME,
                                           AUTHENTICATED_REQUEST)
        self.assertEqual(r, [
            AUTHENTICATED_TEMPLATE_FILENAME,
            TEMPLATE_FILENAME,
        ])

